@@ -20,7 +20,6 @@ class SQLExecute(object):
     sqlscript = None                    # 需要执行的SQL脚本
     SQLMappingHandler = None            # SQL重写处理
     SQLOptions = None                   # 程序处理参数
-    m_Current_RunningSQL = None         # 目前程序运行的当前SQL
     Console = False                     # 屏幕输出Console
     logger = None                       # 日志输出
 
@@ -40,6 +39,9 @@ class SQLExecute(object):
         # 当前Execute的WorkerName
         self.m_Worker_Name = None
 
+        # 程序Spool输出句柄
+        self.spoolfile = None
+
     def set_Worker_Name(self, p_szWorkerName):
         self.m_Worker_Name = p_szWorkerName
 
@@ -50,9 +52,9 @@ class SQLExecute(object):
         self.conn = p_conn
 
     def run(self, statement, p_sqlscript=None):
-        """Execute the sql in the database and return the results. The results
-        are a list of tuples. Each tuple has 4 values
-        (title, rows, headers, status).
+        """
+        返回结果包含5个方面的内容
+        (title, rows, headers, columntypes, status).
         """
         m_SQL_Status = 0             # SQL 运行结果， 0 成功， 1 失败
         m_SQL_ErrorMessage = ""      # 错误日志信息
@@ -60,38 +62,52 @@ class SQLExecute(object):
         # Remove spaces and EOL
         statement = statement.strip()
         if not statement:  # Empty string
-            yield None, None, None, None
+            yield None, None, None, None, None
 
         # 分析SQL语句
-        (ret_bSQLCompleted, ret_SQLSplitResults, ret_SQLSplitResultsWithComments) = SQLAnalyze(statement)
+        (ret_bSQLCompleted, ret_SQLSplitResults,
+         ret_SQLSplitResultsWithComments, ret_SQLFlags) = SQLAnalyze(statement)
         for m_nPos in range(0, len(ret_SQLSplitResults)):
 
             sql = ret_SQLSplitResults[m_nPos]
             m_CommentSQL = ret_SQLSplitResultsWithComments[m_nPos]
-
+            m_SQLFlag = ret_SQLFlags[m_nPos]
+            if "Feature" in m_SQLFlag.keys():
+                m_SQLFeature = m_SQLFlag["Feature"]
+            else:
+                m_SQLFeature = ""
             # 如果打开了回显，并且指定了输出文件，则在输出文件里显示SQL语句
             if self.SQLOptions.get("ECHO").upper() == 'ON' \
                     and len(m_CommentSQL.strip()) != 0 and self.logfile is not None:
                 click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.logfile)
-
-            # 如果运行在脚本模式下，需要在控制台回显SQL
-            if p_sqlscript is not None:
-                click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.Console)
+            if self.SQLOptions.get("ECHO").upper() == 'ON' \
+                    and len(m_CommentSQL.strip()) != 0 and self.spoolfile is not None:
+                # 在spool文件中，不显示spool off的信息，以避免log比对中的不必要内容
+                if not re.match(r'spool\s+.*', m_CommentSQL.strip(), re.IGNORECASE):
+                    click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.spoolfile)
+            # 在logger中显示执行的SQL
             if self.logger is not None:
                 self.logger.info(SQLFormatWithPrefix(m_CommentSQL))
+            # 如果运行在脚本模式下，需要在控制台额外回显SQL
+            # 非脚本模式下，由于是用户自行输入，所以不需要回显输入的SQL
+            if p_sqlscript is not None:
+                click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.Console)
 
             # 如果打开了回显，并且指定了输出文件，且SQL被改写过，输出改写后的SQL
             if self.SQLOptions.get("SQLREWRITE").upper() == 'ON':
                 old_sql = sql
                 sql = self.SQLMappingHandler.RewriteSQL(p_sqlscript, old_sql)
-                if old_sql != sql:
+                if old_sql != sql:    # SQL已经发生了改变
                     if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                        # SQL已经发生了改变
                         click.echo(SQLFormatWithPrefix(
                             "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                    if p_sqlscript is not None:
-                        click.echo(SQLFormatWithPrefix(
-                            "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                    if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
+                        click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                                   file=self.spoolfile)
+                    if self.logger is not None:
+                        self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
+                    click.echo(SQLFormatWithPrefix(
+                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
 
             # 如果是空语句，不在执行
             if len(sql.strip()) == 0:
@@ -102,7 +118,6 @@ class SQLExecute(object):
 
             # 记录命令开始时间
             start = time.time()
-            self.m_Current_RunningSQL = sql
 
             # 检查SQL中是否包含特殊内容
             # 特殊内容都有：
@@ -117,12 +132,15 @@ class SQLExecute(object):
             if sql.find("%lastsqlresult.LastAffectedRows%") != -1:
                 sql = sql.replace("%lastsqlresult.LastAffectedRows%", str(self.LastAffectedRows))
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                    # SQL已经发生了改变
                     click.echo(SQLFormatWithPrefix(
                         "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                if p_sqlscript is not None:
-                    click.echo(SQLFormatWithPrefix(
-                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
+                    click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                               file=self.spoolfile)
+                click.echo(SQLFormatWithPrefix(
+                    "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                if self.logger is not None:
+                    self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
 
             # lastsqlresult.LastSQLResult[X][Y]
             matchObj = re.search(r"%lastsqlresult.LastSQLResult\[(\d+)\]\[(\d+)\]%",
@@ -139,12 +157,15 @@ class SQLExecute(object):
                     m_Result = str(self.LastSQLResult[m_nRow][m_nColumn])
                 sql = sql.replace(m_Searched, m_Result)
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                    # SQL已经发生了改变
                     click.echo(SQLFormatWithPrefix(
                         "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                if p_sqlscript is not None:
-                    click.echo(SQLFormatWithPrefix(
-                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
+                    click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                               file=self.spoolfile)
+                if self.logger is not None:
+                    self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
+                click.echo(SQLFormatWithPrefix(
+                    "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
 
             # ${var}
             bMatched = False
@@ -163,12 +184,16 @@ class SQLExecute(object):
                     break
             if bMatched:
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                    # SQL已经发生了改变
+                    # SQL已经发生了改变, 会将改变后的SQL信息在屏幕上单独显示出来
                     click.echo(SQLFormatWithPrefix(
                         "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                if p_sqlscript is not None:
-                    click.echo(SQLFormatWithPrefix(
-                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
+                    click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                               file=self.spoolfile)
+                if self.logger is not None:
+                    self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
+                click.echo(SQLFormatWithPrefix(
+                    "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
 
             # 执行SQL
             try:
@@ -183,7 +208,7 @@ class SQLExecute(object):
                     else:
                         self.LastAffectedRows = 0
                         self.LastSQLResult = None
-                        yield None, None, None, "Not connected. "
+                        yield None, None, None, None, "Not connected. "
 
                 # 执行正常的SQL语句
                 if cur is not None:
@@ -194,18 +219,27 @@ class SQLExecute(object):
                         cur.execute(sql)
                         rowcount = 0
                         while True:
-                            (title, result, headers, status, m_FetchStatus, m_FetchedRows) = \
+                            (title, result, headers, columntypes, status, m_FetchStatus, m_FetchedRows) = \
                                 self.get_result(cur, rowcount)
                             rowcount = m_FetchedRows
 
                             self.LastAffectedRows = rowcount
                             self.LastSQLResult = copy.copy(result)
-                            yield title, result, headers, status
+
+                            # 检查SQLFlags
+                            # 如果有order字样，对结果进行排序后再输出
+                            if "Order" in m_SQLFlag.keys():
+                                for i in range(1, len(result)):
+                                    for j in range(0, len(result) - i):
+                                        if str(result[j]) > str(result[j + 1]):
+                                            result[j], result[j + 1] = result[j + 1], result[j]
+                            yield title, result, headers, columntypes, status
                             if not m_FetchStatus:
                                 break
 
                         # 记录结束时间
                         end = time.time()
+
                         # 记录SQL日志信息
                         self.Log_Perf(
                             {
@@ -213,6 +247,7 @@ class SQLExecute(object):
                                 "elapsed": end - start,
                                 "SQL": sql,
                                 "SQLStatus": m_SQL_Status,
+                                "Feature": m_SQLFeature,
                                 "ErrorMessage": m_SQL_ErrorMessage,
                                 "thread_name": self.m_Worker_Name
                             }
@@ -228,6 +263,7 @@ class SQLExecute(object):
                                 "elapsed": end - start,
                                 "SQL": sql,
                                 "SQLStatus": m_SQL_Status,
+                                "Feature": m_SQLFeature,
                                 "ErrorMessage": m_SQL_ErrorMessage,
                                 "thread_name": self.m_Worker_Name
                             }
@@ -247,7 +283,7 @@ class SQLExecute(object):
                             else:
                                 self.LastAffectedRows = 0
                                 self.LastSQLResult = None
-                                yield None, None, None, str(e)
+                                yield None, None, None, None, str(e)
                         else:
                             # 发生了其他不明错误
                             raise e
@@ -263,22 +299,26 @@ class SQLExecute(object):
                         print('traceback.format_exc():\n%s' % traceback.format_exc())
                     self.LastAffectedRows = 0
                     self.LastSQLResult = None
-                    yield None, None, None, str(e.message)
-
-            self.m_Current_RunningSQL = None
+                    yield None, None, None, None, str(e.message)
 
             # 如果需要，打印语句执行时间
             end = time.time()
             self.LastElapsedTime = end - start
+
+            if "TimeLimit" in m_SQLFlag.keys():
+                m_TimeLimit = m_SQLFlag["TimeLimit"]
+                m_TimeCost = '%9.0f' % ((end - start) * 1000)
+                if m_TimeLimit < (end - start) * 1000:
+                    raise SQLCliException('LinkoopSQL-Err:: ' 
+                                          'SQL elapsed time [' + m_TimeCost + '] over limit [' + str(m_TimeLimit) + ']')
+                    # yield None, None, None, None, 'SQL elapsed time over limit [' + str(m_TimeLimit) + ']'
+
             if self.SQLOptions.get('TIMING').upper() == 'ON':
                 if sql.strip().upper() not in ('EXIT', 'QUIT'):
-                    yield None, None, None, 'Running time elapsed: %9.2f Seconds' % (end - start)
+                    yield None, None, None, None, 'Running time elapsed: %9.2f Seconds' % (end - start)
             if self.SQLOptions.get('TIME').upper() == 'ON':
                 if sql.strip().upper() not in ('EXIT', 'QUIT'):
-                    yield None, None, None, 'Current clock time  :' + strftime("%Y-%m-%d %H:%M:%S", localtime())
-
-    def get_Current_RunningSQL(self):
-        return self.m_Current_RunningSQL
+                    yield None, None, None, None, 'Current clock time  :' + strftime("%Y-%m-%d %H:%M:%S", localtime())
 
     def get_result(self, cursor, rowcount):
         """Get the current result's data from the cursor."""
@@ -288,6 +328,7 @@ class SQLExecute(object):
         # cursor.description is not None for queries that return result sets,
         # e.g. SELECT.
         result = []
+        columntypes = []
         if cursor.description is not None:
             headers = [x[0] for x in cursor.description]
             status = "{0} row{1} selected."
@@ -296,6 +337,10 @@ class SQLExecute(object):
             if self.SQLOptions.get('TERMOUT').upper() != 'OFF':
                 for row in rowset:
                     m_row = []
+                    # 记录字段类型
+                    if len(columntypes) == 0:
+                        for column in row:
+                            columntypes.append(type(column))
                     for column in row:
                         if str(type(column)).upper().find('OBJECT[]') != -1:
                             m_ColumnValue = "STRUCTURE("
@@ -379,9 +424,9 @@ class SQLExecute(object):
         else:
             status = None
         if self.SQLOptions.get('TERMOUT').upper() == 'OFF':
-            return title, [], headers, status, m_FetchStatus, rowcount
+            return title, [], headers, columntypes, status, m_FetchStatus, rowcount
         else:
-            return title, result, headers, status, m_FetchStatus, rowcount
+            return title, result, headers, columntypes, status, m_FetchStatus, rowcount
 
     # 记录PERF信息，
     def Log_Perf(self, p_SQLResult):
@@ -404,7 +449,7 @@ class SQLExecute(object):
                 self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
                 self.SQLPerfFileHandle.write("Script\tStarted\telapsed\tSQLPrefix\t"
                                              "SQLStatus\tErrorMessage\tthread_name\t"
-                                             "Copies\tFinished\n")
+                                             "Copies\tFinished\tFeature\n")
                 self.SQLPerfFileHandle.close()
 
             if len(str(p_SQLResult["thread_name"]).split('-')) == 3:
@@ -429,7 +474,8 @@ class SQLExecute(object):
                 str(p_SQLResult["SQLStatus"]) + "\t" +
                 "'" + str(p_SQLResult["ErrorMessage"]).replace("\n", " ").replace("\t", "    ") + "'\t" +
                 "'" + str(m_ThreadName) + "'\t" +
-                str(m_Copies) + "'\t" + str(m_CurrentLoop) +
+                str(m_Copies) + "'\t" + str(m_CurrentLoop) + "\t" +
+                "'" + str(p_SQLResult["Feature"]) + "'" +
                 "\n"
             )
             self.SQLPerfFileHandle.flush()
